/*******************************************************************************
 *******************************************************************************
  @file
    embdz19121-led-blink.ino
 
    EMBDZ19121
 

  @date
    04.20.2020

  @company:
    Made Systems, LLC

  @summary:
    This applicaiton code is designed to illustrate how to use the GPIO
    expander I/O on the EMBDZ19121 ESP32 development host microcontroller to
    flash the Status LED

    The GPIO Expander device is accessed via the I2C bus at address 0x20
    

    (c) 2020 Made Systems, LLC. All rights reserved
********************************************************************************
*******************************************************************************/
#include "Wire.h"



static  uint8_t           gpio0;          // port 0 mirror variable of the GPIO expander
static  uint8_t           gpio1;          // port 1 mirror variable of the GPIO expander


    // initialize the hardware GPIO Expander and Serial UART
void setup() {
    Serial.begin(115200);
                    // setup port mirror variables
    gpio0                 = 0b00000000;
    gpio1                 = 0b00000000;

                    // setup the GPIO expander ports on EMBDZ19121
                    // port 0
                    //  - bit 0   = input   GPIO23  (header pin 16)
                    //  - bit 1   = input   GPIO24  (header pin 18)
                    //  - bit 2   = input   GPIO05  (header pin 29)
                    //  - bit 3   = input   GPIO06  (header pin 31)
                    //  - bit 4   = input   GPIO22  (header pin 15)
                    //  - bit 5   = input   GPIO27  (header pin 13)
                    //  - bit 6   = input   GPIO17  (header pin 11)
                    //  - bit 7   = input   GPIO04  (header pin 7)
                    // port 1
                    //  - bit 0   = output  (unused set low)
                    //  - bit 1   = output  (unused set low)
                    //  - bit 2   = output  (unused set low)
                    //  - bit 3   = output  (unused set low)
                    //  - bit 4   = input   GPIO26  (header pin 37)
                    //  - bit 5   = output  GPIO25  (header pin 22) Status LED
                    //  - bit 6   = input   GPIO16  (header pin 36)
                    //  - bit 7   = input   GPIO21  (header pin 38)

                    // setup the I2C bus
    Wire.begin(32, 33, 100000);
                    // initialize GPIO expander port 0 direction register
    Wire.beginTransmission(0x20);
    Wire.write(0x06);     // internal GPIO expander port 0 direction register address
    Wire.write(0xff);
    Wire.endTransmission();
                    // initialize GPIO expander port 1 direction register
    Wire.beginTransmission(0x20);
    Wire.write(0x07);     // internal GPIO expander port 1 direction register address
    Wire.write(0xd0);
    Wire.endTransmission();
                    // initialize GPIO expander port 0 output values
    Wire.beginTransmission(0x20);
    Wire.write(0x02);     // internal GPIO expander port 0 output register address
    Wire.write(gpio0);
    Wire.endTransmission();
                    // initialize GPIO expander port 1 output values
    Wire.beginTransmission(0x20);
    Wire.write(0x03);     // internal GPIO expander port 1 output register address
    Wire.write(gpio1);
    Wire.endTransmission();

    Serial.println("GPIO Expander setup complete...");
}

    // loop forever toggling the Status LED on & off
void loop() {
                    // turn the Status LED on
                    // setup the gpio1 mirror register to set bit 5 high without
                    // changing the other bits in the gpio1 mirror register
    gpio1                 = gpio1 | 0b00100000;
                    // write the port value to the GPIO Expander
    Wire.beginTransmission(0x20);
    Wire.write(0x03);     // internal GPIO expander port 1 output register address
    Wire.write(gpio1);
    Wire.endTransmission();

    Serial.println("Set Status LED on...");

    delay(500);           // wait for 500 mSec

                    // turn the Status LED off
                    // setup the gpio1 mirror register to set bit 5 low without
                    // changing the other bits in the gpio1 mirror register
    gpio1                 = gpio1 & 0b11011111;
                    // write the port value to the GPIO Expander
    Wire.beginTransmission(0x20);
    Wire.write(0x03);     // internal GPIO expander port 1 output register address
    Wire.write(gpio1);
    Wire.endTransmission();

    Serial.println("Set Status LED off...");

    delay(500);           // wait for 500 mSec
}
